@@ -1,23 +1,22 @@
 #include "MainWindow.h"
-#include "NotifySingletons.h"
 #include "MqPublisher.h"
 #include <iostream>
 using namespace std;
 
+// Forward function declarations
+QString baseDir();
+void initLogging (const QString &userIni);
+
 MainWindow::MainWindow(QObject *parent /*= 0*/)
 : QObject(parent)
+, gvApi (true, this)
+, bIsLoggedIn (false)
 , oContacts (this)
-, oInbox (this)
+, oInbox (gvApi, this)
 , checkCounter (0)
 {
-    initLogging ();
-
     qRegisterMetaType<ContactInfo>("ContactInfo");
 
-    GVAccess &webPage = Singletons::getRef().getGVAccess ();
-    // webPage status
-    QObject::connect (&webPage, SIGNAL (status(const QString &, int)),
-                       this   , SLOT   (setStatus(const QString &, int)));
     // Status from contacts object
     QObject::connect (&oContacts, SIGNAL (status   (const QString &, int)),
                        this     , SLOT   (setStatus(const QString &, int)));
@@ -33,8 +32,6 @@ MainWindow::MainWindow(QObject *parent /*= 0*/)
     // Timer tick
     QObject::connect (&mainTimer, SIGNAL(timeout()), this, SLOT(doWork()));
 
-    webPage.setEmitLog (false);
-
     if (!checkParams ()) {
         qApp->quit();
         return;
@@ -43,7 +40,7 @@ MainWindow::MainWindow(QObject *parent /*= 0*/)
     QDateTime dtNow = QDateTime::currentDateTime ();
     QDateTime dtTomorrow = dtNow.addDays (1);
     dtTomorrow.setTime (QTime(0, 0));
-    int sec = (dtTomorrow.toMSecsSinceEpoch() - dtNow.toMSecsSinceEpoch()) / 1000;
+    int sec = dtTomorrow.toTime_t() - dtNow.toTime_t();
     if (sec < 0) sec = 1;
     QTimer::singleShot (sec * 1000, this, SLOT(dailyTimeout()));
     qDebug() << "Daily timer first shot after" << sec << "seconds";
@@ -51,71 +48,16 @@ MainWindow::MainWindow(QObject *parent /*= 0*/)
     QTimer::singleShot (100, this, SLOT(doWork ()));
 }//MainWindow::MainWindow
 
-void
-MainWindow::initLogging ()
-{
-    QString strLogfile = baseDir ();
-    strLogfile += QDir::separator();
-    strLogfile += "notify.log";
-    fLogfile.setFileName (strLogfile);
-    fLogfile.open (QIODevice::WriteOnly | QIODevice::Append);
-}//MainWindow::initLogging
-
-/** Log information to console and to log file
- * This function is invoked from the qDebug handler that is installed in main.
- * @param strText Text to be logged
- * @param level Log level
- */
-void
-MainWindow::log (const QString &strText, int level /*= 10*/)
-{
-    QString strDisp;
-    QRegExp regex("^\"(.*)\"\\s*");
-    if (strText.indexOf (regex) != -1) {
-        strDisp = regex.cap (1);
-    } else {
-        strDisp = strText;
-    }
-
-    QDateTime dt = QDateTime::currentDateTime ();
-    QString strLog = QString("%1 : %2 : %3")
-                     .arg(dt.toString ("yyyy-MM-dd hh:mm:ss.zzz"))
-                     .arg(level)
-                     .arg(strDisp);
-
-    // Send to standard output
-    cout << strLog.toStdString () << endl;
-
-    // Send to log file
-    if (fLogfile.isOpen ()) {
-        QTextStream streamLog(&fLogfile);
-        streamLog << strLog << endl;
-    }
-}//MainWindow::log
-
 void
 MainWindow::setStatus(const QString &strText, int /*timeout = 3000*/)
 {
     qDebug () << strText;
 }//MainWindow::setStatus
 
-QString
-MainWindow::baseDir()
-{
-    QString strBasedir = QDir::homePath();
-    QDir baseDir(strBasedir);
-    if (!baseDir.exists (".qgvdial")) {
-        baseDir.mkdir (".qgvdial");
-    }
-    strBasedir += QDir::separator();
-    strBasedir += ".qgvdial";
-    return strBasedir;
-}//MainWindow::baseDir
-
 void
 MainWindow::doWork ()
 {
-    if (strSelfNumber.isEmpty ()) {
+    if (!bIsLoggedIn) {
         doLogin ();
         return;
     }
@@ -144,10 +86,7 @@ MainWindow::checkParams ()
         QFileInfo fi(args[1]);
         strIni = fi.absoluteFilePath ();
 
-        QString strLogfile = strIni + ".log";
-        fLogfile.close ();
-        fLogfile.setFileName (strLogfile);
-        fLogfile.open (QIODevice::WriteOnly | QIODevice::Append);
+        initLogging (strIni);
     }
 
     if (bUseDefaultIni) {
@@ -291,11 +230,10 @@ MainWindow::checkParams ()
         }
 
         if (bUseProxy) {
-            GVAccess &webPage = Singletons::getRef().getGVAccess ();
-            webPage.setProxySettings (bUseProxy, bUseSystemProxy,
-                                      strProxy, proxyport,
-                                      bProxyAuth,
-                                      strProxyUser, strProxyPass);
+            gvApi.setProxySettings (bUseProxy, bUseSystemProxy,
+                                    strProxy, proxyport,
+                                    bProxyAuth,
+                                    strProxyUser, strProxyPass);
         }
         rv = true;
     } while (0); // End cleanup block (not a loop)
@@ -363,64 +301,58 @@ MainWindow::cipher(const QByteArray &byIn, QByteArray &byOut, bool bEncrypt)
 void
 MainWindow::doLogin ()
 {
-    GVAccess &webPage = Singletons::getRef().getGVAccess ();
-    QVariantList l;
-
     bool bOk = false;
+    AsyncTaskToken *token = new AsyncTaskToken(this);
     do { // Begin cleanup block (not a loop)
-        webPage.setTimeout(60);
+        if (!token) {
+            Q_WARN("Failed to allocate token");
+            break;
+        }
 
-        l += strUser;
-        l += strPass;
+        bOk = connect(token, SIGNAL(completed(AsyncTaskToken*)),
+                      this , SLOT(loginCompleted(AsyncTaskToken*)));
 
-        qDebug ("Logging in...");
+        token->inParams["user"] = strUser;
+        token->inParams["pass"] = strPass;
+
+        Q_DEBUG("Logging in...");
 
         // webPage.workCompleted -> this.loginCompleted
-        if (!webPage.enqueueWork (GVAW_login, l, this,
-                SLOT (loginCompleted (bool, const QVariantList &))))
-        {
-            qCritical ("Login returned immediately with failure!");
+        if (!gvApi.login (token)) {
+            Q_CRIT("Login returned immediately with failure!");
             break;
         }
 
         bOk = true;
     } while (0); // End cleanup block (not a loop)
 
-    if (!bOk)
-    {
-        webPage.setTimeout(20);
+    if (!bOk) {
+        if (token) {
+            token->deleteLater ();
+            token = NULL;
+        }
+
         // Cleanup if any
         strUser.clear ();
         strPass.clear ();
 
-        l.clear ();
-        logoutCompleted (true, l);
-
+        logoutCompleted (NULL);
         qApp->quit ();
     }
 }//MainWindow::doLogin
 
 void
-MainWindow::loginCompleted (bool bOk, const QVariantList &varList)
+MainWindow::loginCompleted (AsyncTaskToken *token)
 {
-    strSelfNumber.clear ();
-    GVAccess &webPage = Singletons::getRef().getGVAccess ();
-    webPage.setTimeout(20);
+    if (!token || (token->status != ATTS_SUCCESS)) {
+        logoutCompleted (NULL);
 
-    if (!bOk)
-    {
-        QVariantList l;
-        logoutCompleted (true, l);
-
-        qCritical ("User login failed");
+        Q_CRIT("User login failed");
         qApp->quit ();
-    }
-    else
-    {
-        qDebug ("User logged in");
+    } else {
+        Q_DEBUG("User logged in");
 
-        // Save the users GV number returned by the login completion
-        strSelfNumber = varList[varList.size()-1].toString ();
+        bIsLoggedIn = true;
 
         oContacts.setUserPass (strUser, strPass);
         oContacts.loginSuccess ();
@@ -428,23 +360,37 @@ MainWindow::loginCompleted (bool bOk, const QVariantList &varList)
 
         QTimer::singleShot (100, this, SLOT(doWork ()));
     }
+
+    if (token) {
+        delete token;
+    }
 }//MainWindow::loginCompleted
 
 void
 MainWindow::doLogout ()
 {
-    GVAccess &webPage = Singletons::getRef().getGVAccess ();
-    QVariantList l;
-    webPage.enqueueWork (GVAW_logout, l, this,
-                         SLOT (logoutCompleted (bool, const QVariantList &)));
+    AsyncTaskToken *token = new AsyncTaskToken(this);
+    connect (token, SIGNAL(completed(AsyncTaskToken*)),
+             this, SLOT(logoutCompleted(AsyncTaskToken*)));
+
+    if (!gvApi.logout (token)) {
+        token->deleteLater ();
+        return;
+    }
 }//MainWindow::doLogout
 
 void
-MainWindow::logoutCompleted (bool, const QVariantList &)
+MainWindow::logoutCompleted (AsyncTaskToken *token)
 {
     // This clears out the table and the view as well
     oContacts.loggedOut ();
     oInbox.loggedOut ();
+
+    bIsLoggedIn = false;
+
+    if (token) {
+        delete token;
+    }
 }//MainWindow::logoutCompleted
 
 void
@@ -456,7 +402,9 @@ MainWindow::getContactsDone (bool bChanges, bool bOK)
         MqPublisher pub(QString("qgvnotify:%1").arg(QHostInfo::localHostName()),
                         m_strMqServer, m_mqPort, m_strMqTopic,
                         this);
-        pub.publish ("contact");
+        pub.publish(QString("contact %1")
+                    .arg(QDateTime::currentDateTime().toUTC().toTime_t())
+                    .toAscii());
     }
 
     startTimer ();
@@ -470,7 +418,9 @@ MainWindow::inboxChanged ()
     MqPublisher pub(QString("qgvnotify:%1").arg(QHostInfo::localHostName()),
                     m_strMqServer, m_mqPort, m_strMqTopic,
                     this);
-    pub.publish ("inbox");
+    pub.publish(QString("inbox %1")
+                .arg(QDateTime::currentDateTime().toUTC().toTime_t())
+                .toAscii());
 
     startTimer ();
 }//MainWindow::inboxChanged
